import time
import torch
import json
import os
from pathlib import Path
from sentence_transformers import SentenceTransformer, util
from typing import Dict, List, Any
from languages import LANGUAGES, get_language_label

# Check device
device = "cuda" if torch.cuda.is_available() else "cpu"
print("Using device:", device)
if device == "cuda":
    print("CUDA device:", torch.cuda.get_device_name(0))


# Start timer
start_time = time.time()

# Load LaBSE multilingual model
model = SentenceTransformer('sentence-transformers/LaBSE')

def calculate_scores(bindings: List[Dict[str, str]]) -> float:
    """Calculate SBERT scores for a list of bindings."""
    source_phrases = [b["source"] for b in bindings]
    target_phrases = [b["target"] for b in bindings]
    
    # Compute embeddings
    source_embeddings = model.encode(source_phrases, convert_to_tensor=True, device=device)
    target_embeddings = model.encode(target_phrases, convert_to_tensor=True, device=device)
    
    # Compute cosine similarity
    cosine_scores = util.cos_sim(source_embeddings, target_embeddings)
    scores = cosine_scores.diag().cpu().numpy()
    
    return float(scores.mean())

def process_json_file(file_path: Path) -> Dict[str, Any]:
    """Process a single JSON file and return scores."""
    with open(file_path, 'r', encoding='utf-8') as f:
        data = json.load(f)
    
    # Extract information from the JSON data
    source_lang = data['sourceLanguage']
    target_lang = data['targetLanguage']
    mode = data['mode']
    precision_mode = data.get('precisionMode', False)
    
    # Convert boolean precision mode to string
    precision = "precision" if precision_mode else "standard"
    
    # Calculate scores
    overall_score = calculate_scores(data['bindings'])
    
    return {
        'source_language': {
            'code': source_lang,
            'label': get_language_label(source_lang)
        },
        'target_language': {
            'code': target_lang,
            'label': get_language_label(target_lang)
        },
        'mode': mode,
        'precision': precision,
        'precision_mode': precision_mode,
        'score': overall_score,
        'num_bindings': len(data['bindings'])
    }

def main():
    # Directory containing the JSON files
    output_dir = Path('sbert-output')
    
    if not output_dir.exists():
        print(f"Error: Directory {output_dir} does not exist")
        return
    
    # Process all JSON files
    results = []
    json_files = list(output_dir.glob('*_bindings.json'))
    
    print(f"Found {len(json_files)} JSON files to process")
    
    for json_file in json_files:
        print(f"\nProcessing {json_file.name}...")
        try:
            result = process_json_file(json_file)
            results.append(result)
            print(f"Score: {result['score']:.3f}")
        except Exception as e:
            print(f"Error processing {json_file.name}: {str(e)}")
    
    # Organize results by mode and precision
    results_by_mode_precision = {
        'simple': {'precision': [], 'standard': []},
        'smart': {'precision': [], 'standard': []},
        'sentence': {'precision': [], 'standard': []}
    }
    
    for result in results:
        mode = result['mode']
        precision = result['precision']
        if mode in results_by_mode_precision and precision in results_by_mode_precision[mode]:
            results_by_mode_precision[mode][precision].append(result)
    
    # Calculate statistics
    stats = {
        'overall': {
            'total_files': len(json_files),
            'successful_processing': len(results),
            'failed_processing': len(json_files) - len(results)
        },
        'by_mode_precision': {}
    }
    
    for mode in ['simple', 'smart', 'sentence']:
        stats['by_mode_precision'][mode] = {}
        for precision in ['precision', 'standard']:
            mode_precision_results = results_by_mode_precision[mode][precision]
            if mode_precision_results:
                scores = [r['score'] for r in mode_precision_results]
                stats['by_mode_precision'][mode][precision] = {
                    'count': len(mode_precision_results),
                    'average_score': sum(scores) / len(scores),
                    'min_score': min(scores),
                    'max_score': max(scores)
                }
    
    # Save detailed results
    output_data = {
        'results': results,
        'statistics': stats,
        'timestamp': time.strftime('%Y-%m-%d %H:%M:%S'),
        'model': 'sentence-transformers/LaBSE',
        'device': device
    }
    
    output_file = output_dir / 'sbert_scores.json'
    with open(output_file, 'w', encoding='utf-8') as f:
        json.dump(output_data, f, indent=2)
    
    print(f"\nResults saved to {output_file}")
    
    # Print summary
    print("\nSummary of scores by mode and precision:")
    for mode in ['simple', 'smart', 'sentence']:
        print(f"\n{mode.capitalize()}:")
        for precision in ['precision', 'standard']:
            if mode in stats['by_mode_precision'] and precision in stats['by_mode_precision'][mode]:
                mode_precision_stats = stats['by_mode_precision'][mode][precision]
                print(f"  {precision.capitalize()}:")
                print(f"    Count: {mode_precision_stats['count']}")
                print(f"    Average score: {mode_precision_stats['average_score']:.3f}")
                print(f"    Min score: {mode_precision_stats['min_score']:.3f}")
                print(f"    Max score: {mode_precision_stats['max_score']:.3f}")

if __name__ == "__main__":
    main()
    
    # End timer and print time taken
    end_time = time.time()
    elapsed_time = end_time - start_time
    print(f"\nTotal elapsed time: {elapsed_time:.2f} seconds")
